@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """module for producing astronomical plots
 
-(c) 2007-2011 Matt Hilton 
+(c) 2007-2012 Matt Hilton 
 
 U{http://astlib.sourceforge.net}
 
@@ -289,11 +289,11 @@ class ImagePlot:
                 y=p['y'][0]
                 ra=p['RA'][0]
                 dec=p['dec'][0]
-                northPoint=dec+p['sizeArcSec']/3600.0
-                eastPoint=ra+p['sizeArcSec']/3600.0
-                sizePix=(p['sizeArcSec']/3600.0)/self.wcs.getPixelSizeDeg()
+                
+                westPoint,eastPoint,southPoint,northPoint=astCoords.calcRADecSearchBox(ra, dec, p['sizeArcSec']/3600.0/2.0)
                 northPix=self.wcs.wcs2pix(ra, northPoint)
                 eastPix=self.wcs.wcs2pix(eastPoint, dec)
+                                
                 edx=eastPix[0]-x
                 edy=eastPix[1]-y
                 ndx=northPix[0]-x
@@ -312,20 +312,18 @@ class ImagePlot:
                 y=p['y'][0]
                 ra=p['RA'][0]
                 dec=p['dec'][0]
-                northPoint=dec+p['sizeArcSec']/3600.0
-                eastPoint=ra+p['sizeArcSec']/3600.0
-                sizePix=(p['sizeArcSec']/3600.0)/self.wcs.getPixelSizeDeg()
+                
+                westPoint,eastPoint,southPoint,northPoint=astCoords.calcRADecSearchBox(ra, dec, p['sizeArcSec']/3600.0/2.0)
                 northPix=self.wcs.wcs2pix(ra, northPoint)
                 eastPix=self.wcs.wcs2pix(eastPoint, dec)
                 edx=eastPix[0]-x
                 edy=eastPix[1]-y
                 ndx=northPix[0]-x
                 ndy=northPix[1]-y
-                #nArrow=patches.Arrow(x, y, ndx, ndy, edgecolor=p['color'], facecolor=p['color'], width=p['width'])
-                eArrow=patches.Arrow(x+edx/2, y, edx/2, edy, edgecolor=p['color'], facecolor=p['color'], width=p['width'])  
-                wArrow=patches.Arrow(x+edx/2, y, -edx/2, edy, edgecolor=p['color'], facecolor=p['color'], width=p['width'])                
+                                
+                eArrow=patches.Arrow(x, y, edx, edy, edgecolor=p['color'], facecolor=p['color'], width=p['width'])  
+                wArrow=patches.Arrow(x, y, -edx, edy, edgecolor=p['color'], facecolor=p['color'], width=p['width'])                
 
-                #self.axes.add_patch(nArrow)
                 self.axes.add_patch(eArrow)
                 self.axes.add_patch(wArrow)
                 
@@ -338,10 +336,8 @@ class ImagePlot:
                 else:
                     scaleLabel="%.0f %s" % (p['sizeArcSec']/3600.0, DEG)
                     
-                pylab.text(x+edx/2, y+0.025*self.data.shape[1], scaleLabel, horizontalalignment='center', 
+                pylab.text(x, y+0.025*self.data.shape[1], scaleLabel, horizontalalignment='center', 
                                 verticalalignment='center', fontsize=p['objLabelSize'], color=p['color'])
-                #pylab.text(x+edx+edx*0.2, y+edy+edy*0.2, "E", horizontalalignment='center', 
-                                #verticalalignment='center', fontsize=p['objLabelSize'], color=p['color'])
                                 
         if self.axesLabels != None:
             pylab.xticks(self.ticsRA[0], self.ticsRA[1], weight='normal', family=self.axesFontFamily, \
@@ -574,39 +570,56 @@ class ImagePlot:
         
         """
         
-        # Work out where the compass is going in WCS coords from the relative location given
-        # Draw the compass 1.5 * the given size in arcsec from that edge of the plot
         if type(location) == str:
-            RADeg, decDeg=self.wcs.getCentreWCSCoords()
-            x, y=self.wcs.wcs2pix(RADeg, decDeg)
-            halfWidthRADeg, halfHeightDecDeg=self.wcs.getHalfSizeDeg()
-            compassDistRADeg=halfWidthRADeg-1.5*sizeArcSec/3600.0
-            compassDistDecDeg=halfHeightDecDeg-1.5*sizeArcSec/3600.0
-            if self.wcs.isFlipped() == True:
-                compassDistRADeg=compassDistRADeg*-1
+            cRADeg, cDecDeg=self.wcs.getCentreWCSCoords()
+            RAMin, RAMax, decMin, decMax=self.wcs.getImageMinMaxWCSCoords()
+            westPoint,eastPoint,southPoint,northPoint=astCoords.calcRADecSearchBox(cRADeg, cDecDeg, sizeArcSec/3600.0/2.0)
+            sizeRADeg=eastPoint-westPoint
+            sizeDecDeg=northPoint-southPoint
+            xSizePix=(sizeArcSec/3600.0)/self.wcs.getXPixelSizeDeg()
+            ySizePix=(sizeArcSec/3600.0)/self.wcs.getYPixelSizeDeg()
+            X=self.data.shape[1]
+            Y=self.data.shape[0]
+            xBufferPix=0.5*xSizePix
+            yBufferPix=0.5*ySizePix
+            cx, cy=self.wcs.wcs2pix(cRADeg, cDecDeg)
             foundLocation=False
-            if location.find("N") != -1:
-                y=y+compassDistDecDeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
-            if location.find("S") != -1:
-                y=y-compassDistDecDeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
-            if location.find("E") != -1:
-                x=x-compassDistRADeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
-            if location.find("W") != -1:
-                x=x+compassDistRADeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
+            x=cy
+            y=cx
+            if self.wcs.isFlipped() == False:              
+                if location.find("N") != -1:
+                    y=Y-2*yBufferPix
+                    foundLocation=True
+                if location.find("S") != -1:
+                    y=yBufferPix
+                    foundLocation=True
+                if location.find("E") != -1:
+                    x=xBufferPix*2
+                    foundLocation=True
+                if location.find("W") != -1:
+                    x=X-xBufferPix
+                    foundLocation=True
+            else:
+                if location.find("S") != -1:
+                    y=Y-2*yBufferPix
+                    foundLocation=True
+                if location.find("N") != -1:
+                    y=yBufferPix
+                    foundLocation=True
+                if location.find("W") != -1:
+                    x=xBufferPix*2
+                    foundLocation=True
+                if location.find("E") != -1:
+                    x=X-xBufferPix
+                    foundLocation=True
             if foundLocation == False:
-                raise Exception, "didn't understand location string for compass (should be e.g. N, S, E, W)."
-        elif type(location) == tuple:
+                raise Exception, "didn't understand location string for scale bar (should be e.g. N, S, E, W)."
+            RADeg, decDeg=self.wcs.pix2wcs(x, y)
+        elif type(location) == tuple or type(location) == list:
             x, y=location
+            RADeg, decDeg=self.wcs.pix2wcs(x, y)
         else:
-            raise Exception, "didn't understand location for compass - should be string or tuple."
-        RADeg, decDeg=self.wcs.pix2wcs(x, y)
-        
-        # Size of compass arrows pixels in plot - converted from arcsec
-        sizePix=(sizeArcSec/3600.0)/self.wcs.getPixelSizeDeg()
+            raise Exception, "didn't understand location for scale bar - should be string or tuple."        
         
         alreadyGot=False
         for p in self.plotObjects:
@@ -618,7 +631,6 @@ class ImagePlot:
                 p['tag']="compass"
                 p['objLabels']=[None]
                 p['symbol']="compass"
-                p['sizePix']=sizePix
                 p['sizeArcSec']=sizeArcSec
                 p['width']=width
                 p['color']=color
@@ -628,7 +640,7 @@ class ImagePlot:
         if alreadyGot == False:
             self.plotObjects.append({'x': [x], 'y': [y], 'RA': [RADeg], 'dec': [decDeg],
                                 'tag': "compass", 'objLabels': [None], 'symbol': "compass", 
-                                'sizePix': sizePix, 'width': width, 'color': color,
+                                'width': width, 'color': color,
                                 'objLabelSize': fontSize, 'sizeArcSec': sizeArcSec})
         self.draw()
 
@@ -655,38 +667,57 @@ class ImagePlot:
         
         """
         
-        # Work out where the scale abr is going in WCS coords from the relative location given
+        # Work out where the scale bar is going in WCS coords from the relative location given
         if type(location) == str:
-            RADeg, decDeg=self.wcs.getCentreWCSCoords()
-            x, y=self.wcs.wcs2pix(RADeg, decDeg)
-            halfWidthRADeg, halfHeightDecDeg=self.wcs.getHalfSizeDeg()
-            compassDistRADeg=halfWidthRADeg-1.2*sizeArcSec/3600.0
-            compassDistDecDeg=halfHeightDecDeg-0.15*halfHeightDecDeg
-            if self.wcs.isFlipped() == True:
-                compassDistRADeg=compassDistRADeg*-1
+            cRADeg, cDecDeg=self.wcs.getCentreWCSCoords()
+            RAMin, RAMax, decMin, decMax=self.wcs.getImageMinMaxWCSCoords()
+            westPoint,eastPoint,southPoint,northPoint=astCoords.calcRADecSearchBox(cRADeg, cDecDeg, sizeArcSec/3600.0/2.0)
+            sizeRADeg=eastPoint-westPoint
+            sizeDecDeg=northPoint-southPoint
+            xSizePix=(sizeArcSec/3600.0)/self.wcs.getXPixelSizeDeg()
+            ySizePix=(sizeArcSec/3600.0)/self.wcs.getYPixelSizeDeg()
+            X=self.data.shape[1]
+            Y=self.data.shape[0]
+            xBufferPix=0.6*ySizePix
+            yBufferPix=0.05*Y
+            cx, cy=self.wcs.wcs2pix(cRADeg, cDecDeg)
             foundLocation=False
-            if location.find("N") != -1:
-                y=y+compassDistDecDeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
-            if location.find("S") != -1:
-                y=y-compassDistDecDeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
-            if location.find("E") != -1:
-                x=x-compassDistRADeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
-            if location.find("W") != -1:
-                x=x+compassDistRADeg/self.wcs.getPixelSizeDeg()
-                foundLocation=True
+            x=cy
+            y=cx
+            if self.wcs.isFlipped() == False:
+                if location.find("N") != -1:
+                    y=Y-1.5*yBufferPix
+                    foundLocation=True
+                if location.find("S") != -1:
+                    y=yBufferPix
+                    foundLocation=True
+                if location.find("E") != -1:
+                    x=xBufferPix
+                    foundLocation=True
+                if location.find("W") != -1:
+                    x=X-xBufferPix
+                    foundLocation=True
+            else:
+                if location.find("S") != -1:
+                    y=Y-1.5*yBufferPix
+                    foundLocation=True
+                if location.find("N") != -1:
+                    y=yBufferPix
+                    foundLocation=True
+                if location.find("W") != -1:
+                    x=xBufferPix
+                    foundLocation=True
+                if location.find("E") != -1:
+                    x=X-xBufferPix
+                    foundLocation=True
             if foundLocation == False:
-                raise Exception, "didn't understand location string for compass (should be e.g. N, S, E, W)."
-        elif type(location) == tuple:
+                raise Exception, "didn't understand location string for scale bar (should be e.g. N, S, E, W)."
+            RADeg, decDeg=self.wcs.pix2wcs(x, y)
+        elif type(location) == tuple or type(location) == list:
             x, y=location
+            RADeg, decDeg=self.wcs.pix2wcs(x, y)
         else:
-            raise Exception, "didn't understand location for compass - should be string or tuple."
-        RADeg, decDeg=self.wcs.pix2wcs(x, y)
-        
-        # Size of scale arrow in pixels on plot - converted from arcsec
-        sizePix=(sizeArcSec/3600.0)/self.wcs.getPixelSizeDeg()
+            raise Exception, "didn't understand location for scale bar - should be string or tuple."
         
         alreadyGot=False
         for p in self.plotObjects:
@@ -698,7 +729,6 @@ class ImagePlot:
                 p['tag']="scaleBar"
                 p['objLabels']=[None]
                 p['symbol']="scaleBar"
-                p['sizePix']=sizePix
                 p['sizeArcSec']=sizeArcSec
                 p['width']=width
                 p['color']=color
@@ -708,7 +738,7 @@ class ImagePlot:
         if alreadyGot == False:
             self.plotObjects.append({'x': [x], 'y': [y], 'RA': [RADeg], 'dec': [decDeg],
                                 'tag': "scaleBar", 'objLabels': [None], 'symbol': "scaleBar", 
-                                'sizePix': sizePix, 'width': width, 'color': color,
+                                'width': width, 'color': color,
                                 'objLabelSize': fontSize, 'sizeArcSec': sizeArcSec})
         self.draw()
                                 
