@@ -691,7 +691,7 @@ def biweightLSFit(dataList, tuningConstant, sigmaCut = None):
                 sigma=stdev(absRes)
                 count=0
                 for item in absRes:
-                    if item>(sigmaClipping*sigma) \
+                    if item>(sigmaCut*sigma) \
                     and len(dataCopy)>2:
                         del dataCopy[count]
                         del res[count]
@@ -710,15 +710,11 @@ def biweightLSFit(dataList, tuningConstant, sigmaCut = None):
             # of residuals as weight
             wData=[]
             for i in range(len(dataCopy)):
-                wData.append([	dataCopy[i][0], 
-                                dataCopy[i][1],
-                                weights[i]])
+                wData.append([dataCopy[i][0], dataCopy[i][1], dataCopy[i][2], weights[i][1]])
+            
             results=weightedLSFit(wData, "weights")
 
-    return {'slope':m,
-            'intercept':c,
-            'slopeError':mSigma,
-            'interceptError':cSigma}
+    return results
     
 #---------------------------------------------------------------------------------------------------
 def cumulativeBinner(data, binMin, binMax, binTotal):
